import * as sp from './seisplotjs_3.1.4-alpha.2_standalone.mjs';


const instaseisUrl = 'http://service.iris.washington.edu/irisws/syngine/1/query?greensfunction&model=ak135f_1s&format=miniseed&units=displacement&sourcedepthinmeters=10000&sourcedistanceindegrees=15&nodata=404'
const ak5sfilename = 'syngine-ak135f_5s-1900-01-01t000000.mseed';
const iaspfilename = 'syngine-iasp91_2s-1900-01-01t000000.mseed'
const akfilename = 'syngine-ak135f_1s-1900-01-01t000000.mseed';
const both_akfilename = 'syngine_ak135_both.mseed';
const timeoutSec=10;

export function filter(seismogram, lpCorner=1.0) {
  let butterworth = sp.filter.createButterworth(
        2, // poles
        sp.filter.LOW_PASS,
        0.0, // low corner
        lpCorner, // high corner
        1 / seismogram.sampleRate, // delta (period)
      );
      //let rmeanSeis = sp.filter.rMean(seismogram);
      let rmeanSeis = seismogram;
      let filteredSeis = sp.filter.applyFilter(butterworth, rmeanSeis);
      return filteredSeis;
}

export function markersFromMSeed3(ms3, sdd) {

}

export function load() {
  const fetchInit = sp.util.defaultFetchInitObj(sp.miniseed.MINISEED_MIME);
  console.log("before fetch")
  return sp.util.doFetchWithTimeout(akfilename, fetchInit, timeoutSec * 1000)
      .then((fetchResponse) => {
        console.log(`got fetch: ${fetchResponse.ok}`)
        if (fetchResponse.ok) {
          return fetchResponse.arrayBuffer().then((ab) => {
            console.log(`arrayBuffer: ${ab.byteLength}`)
            return sp.miniseed.parseDataRecords(ab);
          });
        } else {
          util.log("fetchResponse not ok");
          return [];
        }
      }).then((dataRecords) => {
          const seisList = sp.miniseed.seismogramPerChannel(dataRecords);
          const sddList = seisList.map(s => sp.seismogram.SeismogramDisplayData.fromSeismogram(s));
          const q =  sp.quakeml.createQuakeFromValues("fake_quake", sp.util.isoToDateTime('1900-01-01T00:00:00Z'), 0, 0, 10000);
          sddList.forEach(sdd => {
            sdd.addQuake(q);
            sdd.alignmentTime = q.preferredOrigin.time;
            sdd.addMarker(sp.seismographmarker.createMarkerForOriginTime(q));
          });
          return sddList;
      }).then((sddList) => {
        const wkbjUrl = 'wkbj?model=iasp91&evdepth=10&phases=P,p,S,s&distdeg=15&stadepth=0&format=ms3'
        const wkbjPromise = sp.util.doFetchWithTimeout(wkbjUrl, fetchInit, timeoutSec * 1000)
          .then( response => {
            return response.arrayBuffer();
          }).then(rawBuffer => {
              const dataRecords = sp.mseed3.parseMSeed3Records(rawBuffer);
              return dataRecords;
          }).then(dataRecords => {
            return sp.mseed3.sddPerChannel(dataRecords);
          }).then(wkbjsddList => {
            //const wkbjsddList = wkbjseisList.map(s => sp.seismogram.SeismogramDisplayData.fromSeismogram(filter(s)));
            //const q =  sp.quakeml.createQuakeFromValues("fake_quake", sp.util.isoToDateTime('1970-01-01T00:00:00Z'), 0, 0, 10000);
            wkbjsddList.forEach(sdd => {
              sdd.seismogram = filter(sdd.seismogram);
              for (const q of sdd.quakeList) {
                // assume only one
                sdd.alignmentTime = q.preferredOrigin.time;
                sdd.addMarker(sp.seismographmarker.createMarkerForOriginTime(q));
              }
            });
            return wkbjsddList;
          }).then(wkbjsddList => {
            return sddList.concat(wkbjsddList);
          });
          return wkbjPromise;
      }).then((sddList) => {
        const seismographConfig = new sp.seismographconfig.SeismographConfig();
        seismographConfig.doMarkers=true;
        const divEl = document.querySelector("div.seismograph");
        const orgDisp = new sp.organizeddisplay.OrganizedDisplay(sddList, seismographConfig);
        divEl.appendChild(orgDisp);
        console.log(`after append: ${sddList.length}`)
      });
}
